/// <reference types="titanium" />
import { AbstractNode } from '../nodes/AbstractNode';
import { EventCallback } from '../nodes/ElementNode';
import { AbstractElement } from './AbstractElement';
import { TitaniumElement } from './TitaniumElement';
/**
 * Representation of all elements that are not associated with a Titanium
 * view and thus not directly part of the visual tree.
 *
 * Used for all elements that are not known to the Titanium element registry.
 *
 * As this element is not part of the Titanium visual tree but still can
 * contain Titanium views as its children a special child element handling will
 * be applied. Invisible elements will recursively pass through all of their
 * children to its parent elements. If a Titanium element is reached the child
 * view will be inserted into the Titanium visual tree.
 *
 * Example template:
 *
 *  <View>
 *      <Label>1</Label>
 *      <SomeComponent>  <-- This is an invisible element
 *          <Label>2</Label>
 *      </SomeComponent>
 *      <Label>3</Label>
 *  </View>
 *
 * Resulting visual tree:
 *
 *  View
 *      Label 1
 *      Label 2
 *      Label 3
 *
 * Likewise, a similar handling is applied when setting attributes/events on
 * invisible elements. All attributes/events will be passed along child
 * elements until a visual element is reached and the attributes/events will
 * also be applied to that element.
 */
export declare class InvisibleElement extends AbstractElement {
    get firstVisualChild(): TitaniumElement<Ti.Proxy> | null;
    getAttribute(name: string): any;
    setAttribute(name: string, value: any): void;
    on(eventName: string, handler: EventCallback): void;
    off(eventName: string, handler: EventCallback): void;
    isDetached(): boolean;
    shouldDetachChildren(): boolean;
    insertBefore(newChild: AbstractNode, referenceChild: AbstractNode): void;
    insertIntoVisualTree(child: AbstractElement, atIndex?: number): void;
    private projectToVisualElement;
}
