"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var dom_1 = require("../utils/dom");
var AbstractElement_1 = require("./AbstractElement");
var TitaniumElement_1 = require("./TitaniumElement");
/**
 * Representation of all elements that are not associated with a Titanium
 * view and thus not directly part of the visual tree.
 *
 * Used for all elements that are not known to the Titanium element registry.
 *
 * As this element is not part of the Titanium visual tree but still can
 * contain Titanium views as its children a special child element handling will
 * be applied. Invisible elements will recursively pass through all of their
 * children to its parent elements. If a Titanium element is reached the child
 * view will be inserted into the Titanium visual tree.
 *
 * Example template:
 *
 *  <View>
 *      <Label>1</Label>
 *      <SomeComponent>  <-- This is an invisible element
 *          <Label>2</Label>
 *      </SomeComponent>
 *      <Label>3</Label>
 *  </View>
 *
 * Resulting visual tree:
 *
 *  View
 *      Label 1
 *      Label 2
 *      Label 3
 *
 * Likewise, a similar handling is applied when setting attributes/events on
 * invisible elements. All attributes/events will be passed along child
 * elements until a visual element is reached and the attributes/events will
 * also be applied to that element.
 */
var InvisibleElement = /** @class */ (function (_super) {
    tslib_1.__extends(InvisibleElement, _super);
    function InvisibleElement() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Object.defineProperty(InvisibleElement.prototype, "firstVisualChild", {
        get: function () {
            return dom_1.findSingleVisualElementNoThrow(this);
        },
        enumerable: true,
        configurable: true
    });
    InvisibleElement.prototype.getAttribute = function (name) {
        try {
            var visualElement = dom_1.findSingleVisualElement(this);
            return visualElement.getAttribute(name);
        }
        catch (e) {
            return _super.prototype.getAttribute.call(this, name);
        }
    };
    InvisibleElement.prototype.setAttribute = function (name, value) {
        _super.prototype.setAttribute.call(this, name, value);
        try {
            var visualElement = dom_1.findSingleVisualElement(this);
            visualElement.setAttribute(name, value);
        }
        catch (e) {
            // Do nothing if no visual element was found
        }
    };
    InvisibleElement.prototype.on = function (eventName, handler) {
        try {
            // We do not have event bubbling, so manually delegate the event handler
            // to the underlying visual element.
            var visualElement = dom_1.findSingleVisualElement(this);
            return visualElement.on(eventName, handler);
        }
        catch (e) {
            return _super.prototype.on.call(this, eventName, handler);
        }
    };
    InvisibleElement.prototype.off = function (eventName, handler) {
        try {
            var visualElement = dom_1.findSingleVisualElement(this);
            return visualElement.off(eventName, handler);
        }
        catch (e) {
            return _super.prototype.off.call(this, eventName, handler);
        }
    };
    InvisibleElement.prototype.isDetached = function () {
        return this.hasAttribute('detached') || false;
    };
    InvisibleElement.prototype.shouldDetachChildren = function () {
        return this.hasAttribute('detach-children') || false;
    };
    InvisibleElement.prototype.insertBefore = function (newChild, referenceChild) {
        _super.prototype.insertBefore.call(this, newChild, referenceChild);
        /* istanbul ignore if */
        if (!(newChild instanceof AbstractElement_1.AbstractElement)) {
            return;
        }
        /* istanbul ignore else */
        if (!(this.isDetached() || this.shouldDetachChildren())) {
            this.insertIntoVisualTree(newChild);
        }
        if (this.firstElementChild === newChild && newChild instanceof TitaniumElement_1.TitaniumElement) {
            this.projectToVisualElement(newChild);
        }
    };
    InvisibleElement.prototype.insertIntoVisualTree = function (child, atIndex) {
        var parent = this.parentTemplateElement;
        if (parent === null) {
            return;
        }
        if (child.isDetached() || this.shouldDetachChildren()) {
            return;
        }
        var baseIndex = this.parentElement !== null ? this.parentElement.children.indexOf(this) : 0;
        var insideIndex = atIndex === null || atIndex === undefined ? this.children.indexOf(child) : atIndex;
        parent.insertIntoVisualTree(child, baseIndex + insideIndex);
    };
    InvisibleElement.prototype.projectToVisualElement = function (visualElement) {
        var e_1, _a, e_2, _b, e_3, _c;
        try {
            for (var _d = tslib_1.__values(this.attributes), _e = _d.next(); !_e.done; _e = _d.next()) {
                var _f = tslib_1.__read(_e.value, 2), name = _f[0], value = _f[1];
                visualElement.setAttribute(name, value);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_a = _d.return)) _a.call(_d);
            }
            finally { if (e_1) throw e_1.error; }
        }
        try {
            for (var _g = tslib_1.__values(this.events), _h = _g.next(); !_h.done; _h = _g.next()) {
                var _j = tslib_1.__read(_h.value, 2), eventName = _j[0], handlers = _j[1];
                try {
                    for (var handlers_1 = (e_3 = void 0, tslib_1.__values(handlers)), handlers_1_1 = handlers_1.next(); !handlers_1_1.done; handlers_1_1 = handlers_1.next()) {
                        var handler = handlers_1_1.value;
                        visualElement.on(eventName, handler);
                    }
                }
                catch (e_3_1) { e_3 = { error: e_3_1 }; }
                finally {
                    try {
                        if (handlers_1_1 && !handlers_1_1.done && (_c = handlers_1.return)) _c.call(handlers_1);
                    }
                    finally { if (e_3) throw e_3.error; }
                }
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_h && !_h.done && (_b = _g.return)) _b.call(_g);
            }
            finally { if (e_2) throw e_2.error; }
        }
    };
    return InvisibleElement;
}(AbstractElement_1.AbstractElement));
exports.InvisibleElement = InvisibleElement;
