import { ChildNodeList } from '../ChildNodeList';
export var NodeType;
(function (NodeType) {
    NodeType[NodeType["Element"] = 1] = "Element";
    NodeType[NodeType["Text"] = 3] = "Text";
    NodeType[NodeType["Comment"] = 8] = "Comment";
    NodeType[NodeType["Document"] = 9] = "Document";
})(NodeType || (NodeType = {}));
var AbstractNode = /** @class */ (function () {
    function AbstractNode() {
        this.nodeName = '';
        this.nodeType = null;
        this.parentNode = null;
        this.firstChild = null;
        this._nodeValue = null;
        this._childNodes = null;
        this._nextSibling = this;
        this._previousSibling = this;
    }
    Object.defineProperty(AbstractNode.prototype, "nodeValue", {
        get: function () {
            return null;
        },
        set: function (value) {
            // Will be overriden by comment and text nodes
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractNode.prototype, "parentElement", {
        get: function () {
            var ancestor = this.parentNode;
            return ancestor !== null && ancestor.nodeType === NodeType.Element ? ancestor : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractNode.prototype, "childNodes", {
        get: function () {
            if (!this._childNodes) {
                this._childNodes = new ChildNodeList(this);
            }
            return this._childNodes;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractNode.prototype, "lastChild", {
        get: function () {
            if (this.firstChild) {
                return this.firstChild._previousSibling;
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractNode.prototype, "nextSibling", {
        get: function () {
            if (this.parentNode === null) {
                return null;
            }
            var nextSibling = this._nextSibling;
            if (this.parentNode.firstChild === nextSibling) {
                return null;
            }
            return nextSibling;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractNode.prototype, "previousSibling", {
        get: function () {
            if (this.parentNode === null) {
                return null;
            }
            var previousSibling = this._previousSibling;
            if (this.parentNode.lastChild === previousSibling) {
                return null;
            }
            return previousSibling;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractNode.prototype, "nextElementSibling", {
        get: function () {
            for (var child = this.nextSibling; child !== null; child = child.nextSibling) {
                if (child.nodeType === NodeType.Element) {
                    return child;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AbstractNode.prototype, "previousElementSibling", {
        get: function () {
            for (var child = this.previousSibling; child !== null; child = child.previousSibling) {
                if (child.nodeType === NodeType.Element) {
                    return child;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    AbstractNode.prototype.appendChild = function (childNode) {
        this.insertBefore(childNode, null);
    };
    AbstractNode.prototype.removeChild = function (oldChild) {
        if (oldChild.parentNode !== this) {
            throw new Error("Child node " + oldChild + " not found inside " + this);
        }
        if (this.firstChild === oldChild) {
            this.firstChild = oldChild.nextSibling;
        }
        oldChild.parentNode = null;
        var previousChild = oldChild._previousSibling;
        var nextChild = oldChild._nextSibling;
        if (previousChild !== oldChild) {
            previousChild._nextSibling = nextChild;
            nextChild._previousSibling = previousChild;
            oldChild._previousSibling = oldChild._nextSibling = oldChild;
        }
        this.childNodes.invalidateCache();
    };
    /**
     * @todo Improve performance when node already has a parent
     * @param newNode
     * @param referenceNode
     */
    AbstractNode.prototype.insertBefore = function (newNode, referenceNode) {
        this.ensurePreInsertionValidity(newNode, referenceNode);
        if (newNode.parentNode !== null) {
            newNode.parentNode.removeChild(newNode);
        }
        newNode.parentNode = this;
        if (referenceNode === null) {
            referenceNode = this.firstChild;
        }
        else if (referenceNode === this.firstChild) {
            this.firstChild = newNode;
        }
        if (referenceNode) {
            newNode._previousSibling = referenceNode._previousSibling;
            newNode._nextSibling = referenceNode;
            referenceNode._previousSibling._nextSibling = newNode;
            referenceNode._previousSibling = newNode;
        }
        else {
            this.firstChild = newNode;
        }
        this.childNodes.invalidateCache();
    };
    AbstractNode.prototype.ensurePreInsertionValidity = function (newNode, referenceNode) {
        if (this.nodeType !== NodeType.Element) {
            throw new Error("Tried to insert " + newNode + " into " + this + ". This operation would yield in an invalid DOM tree.");
        }
        if (referenceNode && referenceNode.parentNode !== this) {
            throw new Error("Tried to insert " + newNode + " into " + this + " before " + referenceNode + " but the reference node has a different parent. This operation would yield in an invalid DOM tree.");
        }
    };
    return AbstractNode;
}());
export { AbstractNode };
